@import url('https://fonts.googleapis.com/css2?family=Poppins&family=Roboto+Condensed&family=Roboto+Flex&family=Lobster+Two&family=Madimi+One&display=swap');

body {
    background-color: #ECEFF1;
    font-family: "Roboto Light", sans-serif;
}

h1 {
    font-size: 1.4em;
}

h2 {
    font-size: 1.2em;
}

h3 {
    font-size: 1em;
}
    
h1, h2, h3 {
    font-family: "Roboto Condensed";
    font-weight: normal;
    color: #bf360c;
}

blockquote {
    font-style: italic;
}

a,
a:visited {
    color: #0288d1;
}

.link-button:visited,
.link-button {
    display: block;
    padding: 10px;
    border: 1px solid #ccc;
    border-radius: 4px;
    color: #fff;
    background-color: #00796b;    
    text-decoration: none;
    text-align: center;
    cursor: pointer;
}

.menu select,
#container select,
#container button,
#container input {
    font-family: "Roboto Light", sans-serif;
}

.menu select {
    font-size: 1em;
    background: none;
    border: none;
}

.menu select:focus-visible {
    outline: none;
}

.menu .tool-item {
    padding: 5px 0px;
}

.menu .model-buttons {
    margin-bottom: 10px;
}

.codehilite pre {
    overflow: auto;
    line-height: 150%;
}

#message-counter,
#footer-bar {
    color: gray;
    font-size: 0.7em;
    text-align: right;
    font-family: 'Roboto Condensed';
    margin-top: 8px;
}

#footer-bar a {
    color: gray;
}

#sign-out,
#container {
    margin: auto;
    max-width: 600px;
}

#sign-out {
    text-align: right;
    font-size: .6em;
    margin-bottom: 10px;
}

#header-bar {
    display: flex;
    justify-content: center;
    max-width: 599px;
    margin: 0 auto
}

#header-logo {
    max-width: 100%;
    height: auto;
    border-radius: 4px;
    border: solid 1px black;
    box-shadow: 0 0 10px rgba(0,0,0,0.1);
    margin-bottom: 10px;
}

#container select,
#container input[type="text"],
#container input[type="password"] {
    padding: 10px;
    margin-bottom: 10px;
    border: 1px solid #ccc;
    border-radius: 4px;
}

.message-info {
    margin-bottom: 10px;
    font-size: .8em;
    color: #607D8B;
}

#page-notification {
    border-radius: 4px;
    color: #fff;
    background-color: #00796b;
    box-shadow: 0 0 10px rgba(0,0,0,0.1);
    padding: 10px;
    margin-bottom: 10px;
    font-size: .8em;
}

#page-notification a {
    color: #fff;
}

.message-user {
    color: #00796b;
    white-space: pre-wrap;
}

.message-loading {
    color: gray;
}

.message-loading,
.message-user,
.message-ai {
    margin-bottom: 10px;
}

.codehilite pre {
    padding: 10px;
}

#message-box {
    display: flex;
    justify-content: space-between;
    margin-bottom: 10px;
    margin-top: 10px;
    min-height: 100px;
    position: sticky;
    bottom: 0;
    padding: 10px;
    background-color: #FFFDE7;
    border-radius: 4px;
    box-shadow: 0 0 10px rgba(0,0,0,0.1);
}

.expanded-message-box {
    height: 80vh;
}

#message-container {
    width: 100%;
    display: flex;
    align-items: stretch;
    padding: 10px;
    border: 1px solid #ccc;
    border-radius: 4px;
    background-color: white;
}

#button-container {
    display: flex;
    flex-direction: column;
    align-self: flex-start;
}

textarea#message {
    border: none;
    flex-grow: 1;
    height: 100%;
}

textarea#message:focus {
    outline: none;
}

textarea#message:disabled {
    background: none;
}

button#cancel,
button#expand,
button#send {
    font-size: 1.2em;
    height:100%;
    border: none;
    color: #fff;
    color: #00796b;
    background: none;
    cursor: pointer;
}

button#cancel {
    display: none;
}

button#send:disabled {
    color: #cccccc;
}

input#submit,
button#start {
    width: 100%;
    padding: 10px;
    border: none;
    color: #fff;
    background-color: #00796b;
    border-radius: 4px;
    cursor: pointer;
}

#response {
    margin-bottom: 10px; 
    word-wrap: break-word;
}

#start-text p,
#response p {
    margin: 0px;
    margin-block-end: 10px;
}

#start-text  ul {
    padding-left: 18px;
}

#loading {
    margin-bottom: 10px;
}


.form-group label {
    display: block;
    margin-bottom: 5px;
}

#start-info button {
    width: 100%;
}

#control-box {
    margin-bottom: 10px;
}

#control-box button {
    font-size: 0.6em;
    padding: 5px;
    border: none;
    color: black;
    background-color: #eeeeec;
    border-radius: 4px;
    cursor: pointer;
}

#control-box button:hover {
    text-decoration: underline;
}

#start-text {
    margin-top: 0px;
    margin-bottom: 20px;
    padding: 10px 20px 10px 20px;
    border: 1px solid #ccc;
    border-radius: 4px;
    background-color: white;
    box-shadow: 0 0 10px rgba(0,0,0,0.1);
}

#library {
    margin-left: 10px;
    margin-right: 10px;
}

#example-queries {
    display: flex;
    flex-wrap: wrap;
    justify-content: space-between;
}

.example-query {
    flex: 1 0 30%;
    margin: 10px;
    font-size: 0.6em;
    padding: 10px;
    color: black;
    background: white;
    border: 1px solid #ccc;
    border-radius: 4px;
    cursor: pointer;
}

.example-query:hover {
    color: #fff;
    background-color: #00796b;
}

#user-message {
    border-top: solid 1px;
    padding-top: 10px;
    font-size: 0.6em;
    margin-bottom: -10px;
}

.message-timestamp,
.message-answer-model,
.message-sources {
    font-family: "Roboto Condensed";
    font-size: 0.6em;
    text-align: right;
    color: #90A4AE;
}

.message-ai,
.message-user {
    border-radius: 4px;
    padding: 10px;
    box-shadow: 0 0 10px rgba(0,0,0,0.1);
}

.message-ai {
    margin-right: 10%;
    background-color: #ffffff;
}

.message-user {
    margin-left: 10%;
    background-color: #FFFDE7;
}

.hidden-data {
    display: none;
}

.message-delete {
    float: right;
    padding: 0px;
    border: none;
    background: none;
    display: none;
    font-size: 0.6em;
    color: #90A4AE;    
}

.message-user:hover .message-delete,
.message-ai:hover .message-delete {
    display: block;
}

.pretty-text {
    font-family: "Lobster Two";
    background-color: #ffd600;
    border-radius: 4px;
    padding: 8px;
    float: right;
    margin: 0px 0px 20px 20px!important;
}

.google-scholar-search-results,
.attachment-content {
    margin-bottom: 10px;
    border-radius: 4px;
    padding: 10px;
    max-height: 100px;
    font-size: 0.7em;
    overflow-y: auto;
    background-color: #cfd8dc;
}

.message .message-notification {
    border-top: solid 1px #90A4AE;
    padding-top: 10px;
    padding-bottom: 10px;
    color: #90A4AE;
    font-size: .8em;
}

.message .image-generation {
    margin: 10px 0px 10px 0px;
    border: solid 1px #90A4AE;
    border-radius: 4px;
    padding: 8px;   
}

.message .image-generation img {
    width: 100%;
}

.login-text {
    text-align: center;
    margin-bottom: 30px;
}

.menu small {
    font-family: "Roboto Condensed";
    color: #aaa;
}
